#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "fast_math_functions.h"
#include "main.h"
#include "hd44780.h"
#include "keyer.h"
#include "morse.h"
#include "hal_i2s.h"
#include "configuration.h"
#include "stm32f4xx_hal_flash_ex.h"

volatile enum config_entries current_config_entry = MAGIC_ID + 1;
volatile enum config_mode current_config_mode = CONFIG_PARAMETER;
volatile int  change_val;
int           samples_per_cycle;

struct SavedOptions Options;

static const char *Options_txt[] = {
  "Magic_ID",
  "Decoder Mode",
  "Sidetone Volume",
  "Sidetone Freq",
  "WPM",
  "WPM Farnsworth",
  "Dah/Dit Ratio",
  "Keyer Mode",
  "Paddle Mode",
  "Sidetone Mode",
  "PTT Lead Time",
  "PTT Tail Time",
  "Mem Repeat Time",
  "Beacon Mem: F00",
};

void
default_Options( struct SavedOptions *sv )
{
  int           i;
  sv->magic = 0xa55a;
  sv->id = 0xffff;
#ifdef ENABLE_CW_DECODE
  sv->decoder_mode = DECODER_OFF;
#endif                          // ENABLE_CW_DECODE
  sv->sidetone_volume = 45;     // 45dB
  sv->hz_sidetone = 750;        // 750Hz;
  sv->wpm = 10;                 // 10WPM
  sv->wpm_farnsworth = 18;      // 10/18 Farnsworth
  sv->dah_to_dit_ratio = 300;   // ratio 1:3.0
  sv->keyer_mode = IAMBIC_A;
  sv->paddle_mode = PADDLE_NORMAL;
  sv->sidetone_mode = SIDETONE_ON;
  sv->ptt_lead_time = 10;       // 10ms
  sv->ptt_tail_time = 3;        // 3ms
  sv->memory_repeat_time = 0;   // no repeat
  sv->beacon_memory = 0;        // no beacon
  for( i = 0; i < 12; i++ )
    sv->string_memory[i] = NULL;
}

void
Change_Options( struct SavedOptions *sv, int up_down )
{
  if( current_config_mode == CONFIG_PARAMETER )
  {
    up_down += current_config_entry;
    if( up_down >= CONFIG_END )
      current_config_entry = MAGIC_ID + 1;
    else if( up_down <= MAGIC_ID )
      current_config_entry = CONFIG_END - 1;
    else
      current_config_entry = up_down;
    return;
  }
  switch ( current_config_entry )
  {
#ifdef ENABLE_CW_DECODE
    case DECODER_MODE:
      sv->decoder_mode += up_down;
      if( sv->decoder_mode > DECODER_ONLY )
        sv->decoder_mode = DECODER_ONLY;
      if( sv->decoder_mode < DECODER_OFF )
        sv->decoder_mode = DECODER_OFF;
      TaskNotifyISR( thread_rx_decode, NOTIFY_START_CAPTURE );
      break;
#endif                          // ENABLE_CW_DECODE
    case SIDETONE_VOLUME:
      sv->sidetone_volume += up_down;
      if( sv->sidetone_volume < MIN_VOLUME_DB )
        sv->sidetone_volume = MIN_VOLUME_DB;
      if( sv->sidetone_volume > MAX_VOLUME_DB )
        sv->sidetone_volume = MAX_VOLUME_DB;
      generate_sine_wave(  );
      break;
    case HZ_SIDETONE:
      samples_per_cycle = 5 + 10 * OUTPUT_RATE / sv->hz_sidetone;
      samples_per_cycle /= 10;
      samples_per_cycle -= up_down;
      if( samples_per_cycle < MIN_SINE_SAMPLES )
        samples_per_cycle = MIN_SINE_SAMPLES;
      if( samples_per_cycle > MAX_SINE_SAMPLES )
        samples_per_cycle = MAX_SINE_SAMPLES;
      sv->hz_sidetone = 5 + 10 * OUTPUT_RATE / samples_per_cycle;
      sv->hz_sidetone /= 10;
      generate_sine_wave(  );
      break;
    case WPM:
      sv->wpm += up_down;
      if( sv->wpm < 3 )
        sv->wpm = 3;
      if( sv->wpm > 50 )
        sv->wpm = 50;
      compute_morse_timing(  );
      break;
    case WPM_FARNSWORTH:
      sv->wpm_farnsworth += up_down;
      if( sv->wpm_farnsworth < 3 )
        sv->wpm_farnsworth = 3;
      if( sv->wpm_farnsworth > 50 )
        sv->wpm_farnsworth = 50;
      compute_morse_timing(  );
      break;
    case DAH_TO_DIT_RATIO:
      sv->dah_to_dit_ratio += up_down;
      if( sv->dah_to_dit_ratio < 250 )
        sv->dah_to_dit_ratio = 250;
      if( sv->dah_to_dit_ratio > 350 )
        sv->dah_to_dit_ratio = 350;
      compute_morse_timing(  );
      break;
    case KEYER_MODE:
      sv->keyer_mode += up_down;
      if( sv->keyer_mode < STRAIGHT )
        sv->keyer_mode = STRAIGHT;
      if( sv->keyer_mode > ULTIMATIC )
        sv->keyer_mode = ULTIMATIC;
      break;
    case PADDLE_MODE:
      if( sv->paddle_mode == PADDLE_NORMAL )
        sv->paddle_mode = PADDLE_REVERSE;
      else
        sv->paddle_mode = PADDLE_NORMAL;
      break;
    case SIDETONE_MODE:
      sv->sidetone_mode += up_down;
      if( sv->sidetone_mode > SIDETONE_PADDLE_ONLY )
        sv->sidetone_mode = SIDETONE_PADDLE_ONLY;
      if( sv->sidetone_mode < SIDETONE_OFF )
        sv->sidetone_mode = SIDETONE_OFF;
      break;
    case PTT_LEAD_TIME:
      sv->ptt_lead_time += up_down;
      if( sv->ptt_lead_time < 0 )
        sv->ptt_lead_time = 0;
      if( sv->ptt_lead_time > 1000 )
        sv->ptt_lead_time = 1000;
      break;
    case PTT_TAIL_TIME:
      sv->ptt_tail_time += up_down;
      if( sv->ptt_tail_time < 0 )
        sv->ptt_tail_time = 0;
      if( sv->ptt_tail_time > 1000 )
        sv->ptt_tail_time = 1000;
      break;
    case MEMORY_REPEAT_TIME:
      // one increment is 100ms
      if( sv->memory_repeat_time >= 600 )       // > 1min change to seconds inc
      {
        sv->memory_repeat_time /= 10;
        sv->memory_repeat_time *= 10;
        up_down *= 10;
      }
      if( sv->memory_repeat_time >= 6000 )      // > 10min change to minutes inc
      {
        sv->memory_repeat_time /= 600;
        sv->memory_repeat_time *= 600;
        up_down *= 60;
      }
      if( sv->memory_repeat_time >= 36000 )     // > 1hour change to 5min inc
      {
        sv->memory_repeat_time /= 3000;
        sv->memory_repeat_time *= 3000;
        up_down *= 5;
      }
      sv->memory_repeat_time += up_down;
      if( sv->memory_repeat_time < 0 )
        sv->memory_repeat_time = 0;
      if( sv->memory_repeat_time > 24 * 36000 ) // max 24 hours
        sv->memory_repeat_time = 24 * 36000;
      break;
    case BEACON_MEMORY:
      sv->beacon_memory += up_down;
      if( sv->beacon_memory < 0 )
        sv->beacon_memory = 0;
      if( sv->beacon_memory > 12 )
        sv->beacon_memory = 12;
      break;
    default:
      break;
  }
}

void
display_parameter_value( void )
{
  char          val_str[16];
  int           val;
  hd44780_set_cursor_position( ( current_config_mode == CONFIG_VALUE ), 0 );
  hd44780_putc( 0xa );
  hd44780_set_cursor_position( ( current_config_mode != CONFIG_VALUE ), 0 );
  hd44780_putc( ' ' );
  hd44780_center_string( 0, 1, Options_txt[current_config_entry] );
  val = ( ( u32 * ) & Options )[current_config_entry];
  switch ( current_config_entry )
  {
#ifdef ENABLE_CW_DECODE
    case DECODER_MODE:
      strcpy( val_str,
          val == DECODER_OFF ? "Off" : ( val ==
          DECODER_SPLIT ? "Split" : "Only" ) );
      break;
#endif                          // ENABLE_CW_DECODE
    case SIDETONE_VOLUME:
      sprintf( val_str, "%ddB", val );
      break;
    case HZ_SIDETONE:
      sprintf( val_str, "%dHz", val );
      break;
    case DAH_TO_DIT_RATIO:
      sprintf( val_str, "%d.%02d:1", val / 100, val % 100 );
      break;
    case KEYER_MODE:
      if( val == STRAIGHT )
        strcpy( val_str, "Straight" );
      else if( val == BUG )
        strcpy( val_str, "Bug" );
      else if( val == SINGLE_PADDLE )
        strcpy( val_str, "Single Paddle" );
      else if( val == IAMBIC_A )
        strcpy( val_str, "Iambic A" );
      else if( val == IAMBIC_B )
        strcpy( val_str, "Iambic B" );
      else if( val == ULTIMATIC )
        strcpy( val_str, "Ultimatic" );
      else
        strcpy( val_str, "Unknown" );
      break;
    case PADDLE_MODE:
      strcpy( val_str, val == PADDLE_NORMAL ? "Normal" : "Reverse" );
      break;
    case SIDETONE_MODE:
      if( val == SIDETONE_OFF )
        strcpy( val_str, "Off" );
      else if( val == SIDETONE_ON )
        strcpy( val_str, "On" );
      else if( val == SIDETONE_PADDLE_ONLY )
        strcpy( val_str, "Paddle Only" );
      break;
    case MEMORY_REPEAT_TIME:
      if( val < 600 )
        sprintf( val_str, "%d.%01d s", val / 10, val % 10 );
      else if( val < 6000 )
        sprintf( val_str, "%d min %d s", val / 600, ( val % 600 ) / 10 );
      else if( val < 36000 )
        sprintf( val_str, "%d min", val / 600 );
      else
        sprintf( val_str, "%d h %d min", val / 36000, ( val % 36000 ) / 600 );
      break;
    case BEACON_MEMORY:
      if( val == 0 )
        strcpy( val_str, "- disabled -" );
      else
      {
        hd44780_set_cursor_position( 0, 14 );
        hd44780_putc( '0' + val / 10 );
        hd44780_putc( '0' + val % 10 );
        if( Options.string_memory[val - 1] != NULL )
        {
          strncpy( val_str, Options.string_memory[val - 1], 15 );
          val_str[15] = 0;
        }
        else
          strcpy( val_str, "- empty -" );
      }
      break;
    default:
      sprintf( val_str, "%d", val );
      break;
  }
  hd44780_center_string( 1, 1, val_str );
}

void
compute_morse_timing( void )
{
  int           additional_ms_farnsworth;
  if( Options.wpm_farnsworth > Options.wpm )
  {
    morse_timing.dit = 1200.f / Options.wpm_farnsworth;
    morse_timing.dah =
        12.f * Options.dah_to_dit_ratio / Options.wpm_farnsworth;
    additional_ms_farnsworth =
        60000.f / Options.wpm - 37200.f / Options.wpm_farnsworth;
    morse_timing.intercharacter = 3 * additional_ms_farnsworth / 19;
    morse_timing.interword = 7 * additional_ms_farnsworth / 19;
  }
  else
  {
    morse_timing.dit = 1200.f / Options.wpm;
    morse_timing.dah = 12.f * Options.dah_to_dit_ratio / Options.wpm;
    morse_timing.intercharacter = 0;
    morse_timing.interword = 0;
  }
}

void
generate_sine_wave( void )
{
  int           i, j, amplitude;
  float         x;
  if( Options.sidetone_volume <= MIN_VOLUME_DB )
    return;
  x = ( float ) Options.sidetone_volume / MAX_VOLUME_DB;
  amplitude = x * x * x * x * SHRT_MAX; // logarithmic volume (x^4 is a good approximation)
  samples_per_cycle = 5 + 10 * OUTPUT_RATE / Options.hz_sidetone;
  samples_per_cycle /= 10;
  if( samples_per_cycle & 1 )
  {
    for( i = 0; i < samples_per_cycle; i++ )
      sine_wave[0][2 * i] =
          arm_sin_f32( PI * 2 * i / samples_per_cycle ) * amplitude;
  }
  else                          // speed up calculations for even number of samples
  {
    for( i = 0; i < samples_per_cycle / 2; i++ )
    {
      sine_wave[0][2 * i] =
          arm_sin_f32( PI * 2 * i / samples_per_cycle ) * amplitude;
      sine_wave[0][2 * i + samples_per_cycle] = -sine_wave[0][2 * i];
    }
  }
  for( i = 0; i < samples_per_cycle; i++ )      // store decayed sine waves
    for( j = 1; j < DECAY_STEPS; j++ )
      sine_wave[j][2 * i] =
          sine_wave[0][2 * i] * ( DECAY_STEPS - j ) / DECAY_STEPS;
}

const u8      boot_strap[] = {
  0, 0, 0, 0, 0x1d, 0, 0, 8, 0xc9, 0x43, 0, 8, 0xcb, 0x43, 0, 8, 0xcd, 0x43, 
  0, 8, 0xcf, 0x43, 0, 8, 0xd1, 0x43, 0, 8, 0xdf, 0xf8, 6, 0xe0, 0xde, 0xf8, 
  4, 0xe0, 0x70, 0x47, 0, 0x40, 0, 8, 0xfe, 0xe7
};

static int
program_NV_Options( int id )
{
  int           len, status;
  u32          *s, *d;

  s = ( u32 * ) & Options;
  d = ( u32 * ) ( &__OPTIONS_segment_start__ + id * SO_SIZE );
  len = SO_SIZE / sizeof( u32 );
  HAL_FLASH_Unlock(  );
  while( len-- )
  {
    status = HAL_FLASH_Program( FLASH_TYPEPROGRAM_WORD, ( u32 ) d++, *s++ );
    if( status != HAL_OK )
      break;
  }
  HAL_FLASH_Lock(  );
  return status;
}

static struct SavedOptions *
last_NV_Options( void )
{
  struct SavedOptions *sv =
      ( struct SavedOptions * ) &__OPTIONS_segment_start__;
  if( sv->magic != 0xa55a )
    return NULL;
  while( sv <= ( struct SavedOptions * )
      ( ( u32 ) & __OPTIONS_segment_end__ - SO_SIZE ) && sv->magic == 0xa55a )
    sv = ( struct SavedOptions * ) ( ( u32 ) sv + SO_SIZE );
  // step back to last valid options
  sv = ( struct SavedOptions * ) ( ( u32 ) sv - SO_SIZE );
  return sv;
}

void
load_NV_Options( void )
{
  struct SavedOptions *sv = last_NV_Options(  );
  if( sv )
    memcpy( &Options, sv, sizeof( struct SavedOptions ) );
  else
    default_Options( &Options );
}

static int
erase_sector0( void )
{
  int           len, status;
  u32          *s, *d;
  HAL_FLASH_Unlock(  );
  FLASH_Erase_Sector( FLASH_SECTOR_0, FLASH_VOLTAGE_RANGE_3 );
  status = FLASH_WaitForLastOperation( ( u32 ) FLASH_ERASE_TIMEOUT );
  CLEAR_BIT( FLASH->CR, ( FLASH_CR_SER | FLASH_CR_SNB ) );
  if( status == HAL_OK )
  {
    // restore bootstrap
    d = ( u32 * ) & __IVT_segment_start__;
    s = ( u32 * ) boot_strap;
    len = sizeof( boot_strap ) / sizeof( u32 );
    while( len-- )
    {
      status =
          HAL_FLASH_Program( FLASH_TYPEPROGRAM_WORD, ( u32 ) ( d++ ), *s++ );
      if( status != HAL_OK )
        break;
    }
  }
  HAL_FLASH_Lock(  );
  return status;
}

int
save_NV_Options( void )
{
  int           i, status;
  bool          erase = false;
  struct SavedOptions *sv = last_NV_Options(  );
  // check if options changed from last save
  if( sv && !memcmp( &Options, sv, sizeof( struct SavedOptions ) ) )
    return HAL_OK;
  // local options end of sector reached
  if( sv >= ( struct SavedOptions * )
      ( ( u32 ) & __OPTIONS_segment_end__ - SO_SIZE ) )
    erase = true;
  sv = &Options;
  sv->id++;
  if( erase )
  {
    for( i = 0; i < 12; i++ )
      Options.string_memory[i] = NULL;
    if( ( status = erase_sector0(  ) ) != HAL_OK )
      return status;
  }
  status = program_NV_Options( sv->id );
  return status;
#ifndef ERASE_OPTIONS
#endif
}

static int
program_string( u32 * d )
{
  int           len, status;
  u32          *s;

  s = ( u32 * ) current_string;
  len = ( string_wptr + 1 + 3 ) / sizeof( u32 );
  HAL_FLASH_Unlock(  );
  while( len-- )
  {
    status =
        HAL_FLASH_Program( FLASH_TYPEPROGRAM_WORD, ( u32 ) ( d++ ), *s++ );
    if( status != HAL_OK )
      break;
  }
  HAL_FLASH_Lock(  );
  return status;
}

int
save_current_string( void )
{
  int           i, status;
  bool          erase = false;
  bool          found = false;
  u32          *word;
  char         *str;
  for( word = ( u32 * ) & __STRINGS_segment_start__;
      word < ( u32 * ) & __STRINGS_segment_end__; word++ )
  {
    if( *word == 0xffffffff )
      break;
    str = ( char * ) word;
    if( !memcmp( str, current_string, string_wptr + 1 ) )
    {
      found = true;
      break;
    }
  }
  if( word == ( u32 * ) & __STRINGS_segment_end__ )
    erase = true;
  if( found )
  {
    Options.string_memory[fn_key - 1] = str;
    return HAL_OK;
  }
  if( erase )
  { 
    for( i = 0; i < 12; i++ )
      Options.string_memory[i] = NULL;
    if( ( status = erase_sector0(  ) ) != HAL_OK )
      return status;
    word = ( u32 * ) & __STRINGS_segment_start__;
  }
  if( ( status = program_string( word ) ) != HAL_OK )
    return status;
  Options.string_memory[fn_key - 1] = ( char * ) word;
  return HAL_OK;
}
