/* ----------------------------------------------------------------------
 * Project:	 CMSIS DSP Library
 * Title:	 arm_rfft_fast_init_f32.c
 * Description:	 Split Radix Decimation in Frequency CFFT Floating point processing function
 *
 * $Date:	 23 April 2021
 * $Revision:	 V1.9.0
 *
 * Target Processor: Cortex-M and Cortex-A cores
 * -------------------------------------------------------------------- */

#include "transform_functions.h"
#include "arm_common_tables.h"

#if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || (defined(ARM_TABLE_TWIDDLECOEF_F32_16) && defined(ARM_TABLE_BITREVIDX_FLT_16) && defined(ARM_TABLE_TWIDDLECOEF_RFFT_F32_32))

/**
  @private
  @brief	 Initialization function for the 32pt floating-point real FFT.
  @param[in,out] S  points to an arm_rfft_fast_instance_f32 structure
  @return	 execution status
		   - \ref ARM_MATH_SUCCESS	  : Operation successful
		   - \ref ARM_MATH_ARGUMENT_ERROR : an error is detected
 */

static arm_status
arm_rfft_32_fast_init_f32( arm_rfft_fast_instance_f32 * S )
{
  arm_status    status;

  if( !S )
    return ARM_MATH_ARGUMENT_ERROR;

  status = arm_cfft_init_f32( &( S->Sint ), 16 );
  if( status != ARM_MATH_SUCCESS )
  {
    return ( status );
  }

  S->fftLenRFFT = 32U;
  S->pTwiddleRFFT = ( float32_t * ) twiddleCoef_rfft_32;

  return ARM_MATH_SUCCESS;
}
#endif

#if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || (defined(ARM_TABLE_TWIDDLECOEF_F32_32) && defined(ARM_TABLE_BITREVIDX_FLT_32) && defined(ARM_TABLE_TWIDDLECOEF_RFFT_F32_64))

/**
  @private
  @brief	 Initialization function for the 64pt floating-point real FFT.
  @param[in,out] S  points to an arm_rfft_fast_instance_f32 structure
  @return	 execution status
		   - \ref ARM_MATH_SUCCESS	  : Operation successful
		   - \ref ARM_MATH_ARGUMENT_ERROR : an error is detected
 */

static arm_status
arm_rfft_64_fast_init_f32( arm_rfft_fast_instance_f32 * S )
{

  arm_status    status;

  if( !S )
    return ARM_MATH_ARGUMENT_ERROR;

  status = arm_cfft_init_f32( &( S->Sint ), 32 );
  if( status != ARM_MATH_SUCCESS )
  {
    return ( status );
  }
  S->fftLenRFFT = 64U;

  S->pTwiddleRFFT = ( float32_t * ) twiddleCoef_rfft_64;

  return ARM_MATH_SUCCESS;
}
#endif

#if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || (defined(ARM_TABLE_TWIDDLECOEF_F32_64) && defined(ARM_TABLE_BITREVIDX_FLT_64) && defined(ARM_TABLE_TWIDDLECOEF_RFFT_F32_128))

/**
  @private
  @brief	 Initialization function for the 128pt floating-point real FFT.
  @param[in,out] S  points to an arm_rfft_fast_instance_f32 structure
  @return	 execution status
		   - \ref ARM_MATH_SUCCESS	  : Operation successful
		   - \ref ARM_MATH_ARGUMENT_ERROR : an error is detected
 */

static arm_status
arm_rfft_128_fast_init_f32( arm_rfft_fast_instance_f32 * S )
{

  arm_status    status;

  if( !S )
    return ARM_MATH_ARGUMENT_ERROR;

  status = arm_cfft_init_f32( &( S->Sint ), 64 );
  if( status != ARM_MATH_SUCCESS )
  {
    return ( status );
  }
  S->fftLenRFFT = 128;

  S->pTwiddleRFFT = ( float32_t * ) twiddleCoef_rfft_128;

  return ARM_MATH_SUCCESS;
}
#endif

/**
  @brief	 Initialization function for the floating-point real FFT.
  @param[in,out] S	 points to an arm_rfft_fast_instance_f32 structure
  @param[in]	 fftLen	 length of the Real Sequence
  @return	 execution status
		   - \ref ARM_MATH_SUCCESS	  : Operation successful
		   - \ref ARM_MATH_ARGUMENT_ERROR : <code>fftLen</code> is not a supported length

  @par		 Description
		   The parameter <code>fftLen</code> specifies the length of RFFT/CIFFT process.
  @par
		   This Function also initializes Twiddle factor table pointer and Bit reversal table pointer.
 */

arm_status
arm_rfft_fast_init_f32( arm_rfft_fast_instance_f32 * S, uint16_t fftLen )
{
  typedef       arm_status( *fft_init_ptr ) ( arm_rfft_fast_instance_f32 * );
  fft_init_ptr  fptr = 0x0;

  switch ( fftLen )
  {
#if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || (defined(ARM_TABLE_TWIDDLECOEF_F32_64) && defined(ARM_TABLE_BITREVIDX_FLT_64) && defined(ARM_TABLE_TWIDDLECOEF_RFFT_F32_128))
    case 128U:
      fptr = arm_rfft_128_fast_init_f32;
      break;
#endif
#if !defined(ARM_DSP_CONFIG_TABLES) || defined(ARM_ALL_FFT_TABLES) || (defined(ARM_TABLE_TWIDDLECOEF_F32_32) && defined(ARM_TABLE_BITREVIDX_FLT_32) && defined(ARM_TABLE_TWIDDLECOEF_RFFT_F32_64))
    case 64U:
      fptr = arm_rfft_64_fast_init_f32;
      break;
#endif
    default:
      return ARM_MATH_ARGUMENT_ERROR;
  }
  if( !fptr )
    return ARM_MATH_ARGUMENT_ERROR;
  return fptr( S );
}
